#ifdef LINUX


/*
 Linux (POSIX) implementation of _kbhit().
 Morgan McGuire, morgan@cs.brown.edu
 */
//#include <asm/ioctls.h>
#include <sys/ioctl.h> 
#include <stdio.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <termios.h>
//#include <stropts.h>
#include <unistd.h> 
#include <stdlib.h> 
#include <string.h> 
#include <errno.h> 
#include <sys/types.h>
#include "linux-support.h"

int _kbhit() {
#if 0
	 static const int STDIN = 0;
	 static bool initialized = false;

	 if (! initialized) {
		 // Use termios to turn off line buffering
		 termios term;
		 tcgetattr(STDIN, &term);
		 term.c_lflag &= ~ICANON;
		 tcsetattr(STDIN, TCSANOW, &term);
		 setbuf(stdin, NULL);
		 initialized = true;
	 }

	 int bytesWaiting;
	 ioctl(STDIN, FIONREAD, &bytesWaiting);
	 return bytesWaiting;
#else
	struct termios oldterm, newterm;
	int ch, oldflags;

	tcgetattr( STDIN_FILENO, &oldterm );
	newterm = oldterm;
	newterm.c_lflag &= ~( ICANON | ECHO );
	tcsetattr( STDIN_FILENO, TCSANOW, &newterm );
	oldflags = fcntl( STDIN_FILENO, F_GETFL, 0 );
	fcntl( STDIN_FILENO, F_SETFL, oldflags | O_NONBLOCK );

	ch = getchar();

	tcsetattr( STDIN_FILENO, TCSANOW, &oldterm );
	fcntl( STDIN_FILENO, F_SETFL, oldflags );

	if ( ch != EOF )
	{
		ungetc( ch, stdin );
		return 1;
	}
	return 0;
#endif
 }

// Abstract:   split a path into its parts 
 // Parameters: Path: Object to split 
 //             Drive: Logical drive , only for compatibility , not considered 
 //             Directory: Directory part of path 
 //             Filename: File part of path 
 //             Extension: Extension part of path (includes the leading point) 
 // Returns:    Directory Filename and Extension are changed 
 // Comment:    Note that the concept of an extension is not available in Linux, 
 //             nevertheless it is considered 
 
void _splitpath(const char* Path,char* Drive,char* Directory,char* 
 Filename,char* Extension) 
 { 
   char *tempBuffer = new char[PATH_MAX];
   char *CopyOfPath = (char*) Path; 
   int Counter = 0; 
   int Last = 0; 
   int Rest = 0; 
 
  // no drives available in linux . 
   // extensions are not common in linux 
   // but considered anyway 
   Drive = NULL; 
   if(Filename==NULL)
	   Filename=tempBuffer;
   if(Extension==NULL)
	   Extension=tempBuffer;
   if(Directory==NULL)
	   Directory=tempBuffer;
   if(Path==NULL)
	   Path=tempBuffer;
 
  while(*CopyOfPath != '\0') 
     { 
       // search for the last slash 
       while(*CopyOfPath != '/' && *CopyOfPath != '\0') 
         { 
           CopyOfPath++; 
           Counter++; 
         } 
       if(*CopyOfPath == '/') 
         { 
           CopyOfPath++; 
          Counter++; 
           Last = Counter; 
         } 
       else 
           Rest = Counter - Last; 
     } 
   // directory is the first part of the path until the 
   // last slash appears 
   strncpy(Directory,Path,Last); 
   // strncpy doesnt add a '\0' 
   Directory[Last] = '\0'; 
   // Filename is the part behind the last slahs 
   strcpy(Filename,CopyOfPath -= Rest); 
   // get extension if there is any 
   while(*Filename != '\0') 
   { 
     // the part behind the point is called extension in windows systems 
     // at least that is what i thought apperantly the '.' is used as part 
     // of the extension too . 
     if(*Filename == '.') 
       { 
         while(*Filename != '\0') 
         { 
           *Extension = *Filename; 
           Extension++; 
           Filename++; 
         } 
       } 
       if(*Filename != '\0') 
         {Filename++;} 
   } 
   *Extension = '\0'; 
   return; 
 
} 
 
// Abstract:   Make a path out of its parts 
 // Parameters: Path: Object to be made 
 //             Drive: Logical drive , only for compatibility , not considered 
 //             Directory: Directory part of path 
 //             Filename: File part of path 
 //             Extension: Extension part of path (includes the leading point) 
 // Returns:    Path is changed 
 // Comment:    Note that the concept of an extension is not available in Linux, 
 //             nevertheless it is considered 
 
void _makepath(char* Path,const char* Drive,const char* Directory, 
        const char* File,const char* Extension) 
 {   
   while(Drive != NULL && *Drive != '\0') 
   { 
     *Path = *Drive; 
     Path++; 
     Drive++; 
   } 
   while(Directory != NULL && *Directory != '\0') 
   { 
     *Path = *Directory; 
     Path ++; 
     Directory ++; 
   } 
   while(File!=NULL && *File != '\0') 
   { 
     *Path = *File; 
     Path ++; 
     File ++; 
   } 
   while(
	Extension != NULL &&
	*Extension != '\0')
   { 
     *Path = *Extension; 
     Path ++; 
     Extension ++; 
   } 
   *Path = '\0'; 
   return; 
 
} 
 
// Abstract:   Change the current working directory 
 // Parameters: Directory: The Directory which should be the working directory. 
 // Returns:    0 for success , other for error 
 // Comment:    The command doesnt fork() , thus the directory is changed for 
 //             The actual process and not for a forked one which would be true 
 //             for system("cd DIR"); 
 
int _chdir(const char* Directory) 
 { 
   chdir(Directory); 
   return 0; 
 
} 
 
#endif 
